/*******************************************************************
 *
 *    DESCRIPTION:	ata_if.h
 *
 *    AUTHOR:		Mark de Jong
 *
 *    HISTORY:    
 *
 *******************************************************************/
#ifndef ATA_IF_H
#define ATA_IF_H

#define ATA_OK				0x0000
#define ATA_ERROR			0x0100
#define ATA_ERROR_TIME_OUT	0x0200
#define ATA_MALLLOC_ERROR	0x0400

#define ATA_AVAILABLE		0x01
#define ATA_NOT_AVAILABLE	0x00

#define FAT_12				0x12
#define FAT_16				0x16
#define FAT_32				0x32

#define DRIVE_C				0x00
#define DRIVE_D				0x01
#define DRIVE_E				0x02
#define DRIVE_F				0x03

#define ATA_TYPE			0x01
#define ATAPI_TYPE			0x02

#define ATA_BS_AND_BPB_SECTOR	1

typedef struct
{
	unsigned int  available;	
	unsigned char model_name[41];
	unsigned char model_serial_number[21];
	unsigned char firmware_revision[9];
	unsigned int  number_of_heads;
	unsigned int  number_of_cylinders;
	unsigned long number_of_sectors;
	unsigned int  number_of_sectors_per_track;
	unsigned int  bytes_per_sector;
	unsigned int  capabilities;
	unsigned int  configuration;
} ata_ide_drive_info_type;

typedef struct
{
	unsigned int  available;	
	unsigned char model_name[41];
	unsigned char model_serial_number[21];
	unsigned char firmware_revision[9];
	unsigned int  capabilities;
	unsigned int  configuration;
} atapi_ide_drive_info_type;

typedef struct
{
	unsigned char OEMName[8];
	unsigned char VolumeName[12];
	unsigned long VolumeID;
	unsigned long VolumeSize;
	unsigned int  FatType;
	unsigned int  FatSize;
	unsigned int  BytesPerSector;
	unsigned int  SectorsPerCluster;
	unsigned long TotalSectors;
	unsigned long DataSectors;
	unsigned long ReservedSectors;
	unsigned long CountOfClusters;
	unsigned long FirstFatSector;
	unsigned long FirstFatCluster;
	unsigned long SecondFatSector;
	unsigned long RootDirSectors;
	unsigned long FirstDataSector;
	unsigned long FirstRootDirSector;
	unsigned long FirstVolumeSector;
} volume_info_type;

#include "ata_mbr.inc"

typedef struct
{
	unsigned int				master_type;
	ata_ide_drive_info_type		master;
	MBR							master_mbr;
	BS_AND_BPB					master_bs_and_bpb;
	volume_info_type			master_volume[4];
	atapi_ide_drive_info_type	master_atapi;

	unsigned int				slave_type;
	ata_ide_drive_info_type 	slave;
	MBR							slave_mbr;
	BS_AND_BPB					slave_bs_and_bpb;
	volume_info_type			slave_volume[4];
	atapi_ide_drive_info_type	slave_atapi;
} ata_ide_info_type;

#include "ata_io.h"
#include "ata_ide.h"
#include "ata_mbr.h"
#include "ata_fat.h"
#include "ata_dir.h"
#include "ata_atapi.h"

#endif // ATA_IF_H