/*******************************************************************
 *
 *    DESCRIPTION:	ata_io.h
 *
 *    AUTHOR:		Mark de Jong
 *
 *    HISTORY:    
 *
 *******************************************************************/

#ifndef ATA_IO_H
#define ATA_IO_H

#define ATA_MASTER_DRIVE	0x01
#define ATA_SLAVE_DRIVE		0x02

#define	ATA_TIME_OUT		31000		// time-out of 31 seconds

// ATA Section

// ATA command registers
// b0	=>	ATA_A0
// b1	=>	ATA_A1
// b2	=>	ATA_A2
// b3	=>	ATA_CS0
// b4	=>	ATA_CS1

#define	ATA_REG_CMD_MASK			0x1F

// read registers
#define	ATA_DATA_REG				0x10
#define ATA_ERROR_REG				0x11
#define ATA_SECTOR_COUNT_REG		0x12
#define ATA_SECTOR_NUMBER_REG		0x13
#define ATA_BLOCK_ADDRESS_1_REG		0x13
#define ATA_CYLINDER_REG			0x14
#define ATA_BLOCK_ADDRESS_2_REG		0x14
#define ATA_CYLINDER_1_REG			0x15
#define ATA_BLOCK_ADDRESS_3_REG		0x15
#define ATA_DRIVE_HEAD_REG			0x16
#define ATA_BLOCK_ADDRESS_4_REG		0x16
#define ATA_STATUS_REG				0x17
#define ATA_ALT_STATUS_REG			0x0E
#define ATA_ADDRESS_REG				0x0F

// write register
#define	ATA_DATA_REG				0x10
#define ATA_FEATURE_REG				0x11
//#define ATA_SECTOR_COUNT_REG		0x12
//#define ATA_SECTOR_NUMBER_REG		0x13
//#define ATA_BLOCK_ADDRESS_1_REG	0x13
//#define ATA_CYLINDER_REG			0x14
//#define ATA_BLOCK_ADDRESS_2_REG	0x14
//#define ATA_CYLINDER_1_REG		0x15
//#define ATA_BLOCK_ADDRESS_3_REG	0x15
//#define ATA_DRIVE_HEAD_REG		0x16
//#define ATA_BLOCK_ADDRESS_4_REG	0x16
#define ATA_COMMAND_REG				0x17
#define ATA_CONTROL_REG				0x0E
//#define ATA_ADDRESS_REG			0x0F

// ATA error bits from reg ATA_ERROR_REG
#define	ATA_ERROR_AMNF				0x01
#define	ATA_ERROR_TK0NF				0x02
#define	ATA_ERROR_ABRT				0x04
#define	ATA_ERROR_MCR				0x08
#define	ATA_ERROR_IDNF				0x10
#define	ATA_ERROR_MC				0x20
#define	ATA_ERROR_UNC				0x40
#define	ATA_ERROR_BBK				0x80

// ATA status bits from reg ATA_STATUS_REG
#define	ATA_STATUS_ERR				0x01
#define	ATA_STATUS_IDX				0x02
#define	ATA_STATUS_CORR				0x04
#define	ATA_STATUS_DRQ				0x08
#define	ATA_STATUS_DSC				0x10
#define	ATA_STATUS_DF				0x20
#define	ATA_STATUS_DRDY				0x40
#define	ATA_STATUS_BSY				0x80

// ATA control bits from reg ATA_CONTROL_REG
#define	ATA_CONTROL_IEN				0x02
#define	ATA_CONTROL_SRST			0x04

// ATA address bits from reg ATA_ADDRESS_REG
#define ATA_ADDRESS_DS_MASK			0x03
#define	ATA_ADDRESS_HS_MASK			0x07
#define	ATA_ADDRESS_HS_SHIFT		0x02
#define	ATA_ADDRESS_WTG				0x40

// ATA commands
#define ATA_CMD_EXEC_DRIVE_DIAG		0x90
#define ATA_CMD_IDENT_DRIVE			0xEC
#define ATA_CMD_INIT_PARAM			0x91
#define ATA_CMD_READ_SECTORS		0x20
#define ATA_CMD_READ_SECTORS_NO_VER	0x21
#define ATA_CMD_READ_VER_SECTORS	0x40
#define ATA_CMD_SEEK				0x70
#define ATA_CMD_WRITE_SECTORS		0x30

// ATA Drive Identify Result
_packed typedef struct
{
	unsigned int	configuration;
	unsigned int	number_of_cyl;
	unsigned int	reserved_1;
	unsigned int	number_of_head;
	unsigned int	obsolete_1;
	unsigned int	obsolete_2;
	unsigned int	sectors_per_track;
	unsigned int	vendor_specific_1[3];
	unsigned char	serial_number[20];
	unsigned int	obsolete_3;
	unsigned int	obsolete_4;
	unsigned int	number_of_ecc_bytes;
	unsigned char	firmware_revision[8];
	unsigned char	model_name[40];
	unsigned int	sectors_per_interrupt;
	unsigned int	reserved_2;
	unsigned int	capabilities;
	unsigned int	reserved_3;
	unsigned int	timing_mode_pio;
	unsigned int	timing_mode_dma;
	unsigned int	reserved_4;
	unsigned int	number_of_cylinders;
	unsigned int	number_of_heads;
	unsigned int	number_of_sectors_per_track;
	unsigned long	number_of_sectors[2];
	unsigned int	number_of_sectors_per_interrupt;
	unsigned long	number_of_lba_sectors[2];
	unsigned int	single_dma_modes;
	unsigned int	multiple_dma_modes;
	unsigned int	reserved_5;
	unsigned int	reserved_6;
	unsigned int	reserved_7;
	unsigned int	reserved_8;
	unsigned int	reserved_9;
	unsigned int	reserved_10[59];
	unsigned int	vendor_specific_2[32];
	unsigned int	reserved_11[96];
} ATA_IDENTIFY_RESULT;


// ATAPI Section

// ATAPI command registers
// b0	=>	ATA_A0
// b1	=>	ATA_A1
// b2	=>	ATA_A2
// b3	=>	ATA_CS0
// b4	=>	ATA_CS1

#define	ATAPI_REG_CMD_MASK			0x1F

// read registers
#define	ATAPI_DATA_REG				0x10
#define ATAPI_ERROR_REG				0x11
#define ATAPI_INT_REG				0x12
#define ATAPI_BYTE_COUNT_L_REG		0x14
#define ATAPI_BYTE_COUNT_H_REG		0x15
#define ATAPI_DRIVE_SELECT_REG		0x16
#define ATAPI_STATUS_REG			0x17


// write register
#define	ATAPI_DATA_REG				0x10
#define ATAPI_FEATURE_REG			0x11
#define ATAPI_SECTOR_COUNT_REG		0x12
#define ATAPI_SECTOR_NR_REG			0x13
#define ATAPI_BYTE_COUNT_L_REG		0x14
#define ATAPI_BYTE_COUNT_H_REG		0x15
#define ATAPI_DRIVE_SELECT_REG		0x16
#define ATAPI_COMMAND_REG			0x17
#define ATAPI_CONTROL_REG			0x0E

// ATAPI commands
#define ATAPI_CMD_EXEC_DRIVE_DIAG	0x90
#define ATAPI_CMD_IDENT_DRIVE		0xA1
#define ATAPI_CMD_PACKET			0xA0
#define ATAPI_CMD_SOFT_RESET		0x08

// ATAPI Door actions
#define ATAPI_DOOR_CLOSE			0x03
#define ATAPI_DOOR_OPEN				0x02

// ATAPI Audio
#define ATAPI_SCAN_FORWARD			0x01
#define ATAPI_SCAN_REWIND			0x02

// ATAPI Drive Identify Result
_packed typedef struct
{
	unsigned int	configuration;
	unsigned int	reserved_1[9];
	unsigned char	serial_number[20];
	unsigned int	reserved_2[3];
	unsigned char	firmware_revision[8];
	unsigned char	model_name[40];
	unsigned long	reserved_3;
	unsigned int	capabilities;
	unsigned int	reserved_4;
	unsigned int	timing_mode_pio;
	unsigned int	timing_mode_dma;
	unsigned int	reserved_411;
	unsigned int	number_of_cylinders;
	unsigned int	number_of_heads;
	unsigned int	number_of_sectors_per_track;
	unsigned long	number_of_sectors[2];
	unsigned int	number_of_sectors_per_interrupt;
	unsigned long	number_of_lba_sectors[2];
	unsigned int	single_dma_modes;
	unsigned int	multiple_dma_modes;
	unsigned int	reserved_5;
	unsigned int	reserved_6;
	unsigned int	reserved_7;
	unsigned int	reserved_8;
	unsigned int	reserved_9;
	unsigned int	reserved_10[59];
	unsigned int	vendor_specific_2[32];
	unsigned int	reserved_11[96];
} ATAPI_IDENTIFY_RESULT;

// initialize
unsigned int ata_io_init( void );

// byte low-level I/O routines
unsigned int ata_io_read_byte( unsigned int drive, unsigned int addr, unsigned char *data );
unsigned int ata_io_write_byte( unsigned int drive, unsigned int addr, unsigned char data );

// word low-level I/O routines
unsigned int ata_io_read_word( unsigned int drive, unsigned int addr, unsigned int *data );
unsigned int ata_io_write_word( unsigned int drive, unsigned int addr, unsigned int data );

// command access routines
unsigned int ata_io_write_command( unsigned int drive, unsigned int command );
unsigned int ata_io_identify_drive( unsigned int drive, unsigned int addr, unsigned int *data );
unsigned int ata_io_read_status( unsigned int drive );

// sector access routines
unsigned int ata_io_read_sector( unsigned int drive, unsigned int sector, unsigned int *sector );
unsigned int ata_io_write_sector( unsigned int drive, unsigned int sector, unsigned int *sector );

// error access routines
unsigned int ata_io_read_error( unsigned int drive );

#endif // ATA_IO_H
